ibrary(tidyverse)

 normalit <- function(m){
   (m - min(m))/(max(m) - min(m))
 }

#|Code | Species | Common name   | Use |
#|---|---|---|
#| arath  | Arabidopsis Thaliana  | Arabidopsis  | train |
#| braol  | Brassica oleracea  | Broccoli  |train  |
#| cansa  | Cannabis sativa  | hemp  | train |
#| cerri | Ceratopteris richardii | C-fern | train |
#| chlre  | Chlamydomonas reinhardtii  | Chlamydomonas   | train |
#| chqui  | Chenopodium quinoa  | Quinoa  | train |
#| orysj  |  Oryza sativa var. japonica | Rice  | train |
#| selml | Selaginella moellendorffii | Selaginella   | train |
#| sollc  | Solanum  lycopersicum | Tomato  | train |
#| wheat  | Triticum  Aestivum | Wheat  | train |
#| soybn | Glycine max | Soybean | train |
#| cocnu | Cocos nucifera | Coconut | train |
#| maize | MAIZE | maize | test |



data_path <- "protein_identification/identified_elutions"
virNOG_elut_files <- dir(data_path, pattern = "*virNOG_elut.tidy") # Get filenames

virNOG_elut <- data_frame(filename = virNOG_elut_files) %>%
               mutate(file_contents = map(filename, ~ read_csv(file.path(data_path, .)))) %>%
               unnest()

ppm_elut_files <- dir(data_path, pattern = "*virNOG_byfraction_PPMelut.tidy") # Get filenames

ppm_elut <- data_frame(filename = ppm_elut_files) %>%
               mutate(file_contents = map(filename, ~ read_csv(file.path(data_path, .)))) %>%
               unnest()

ppm_elut <- ppm_elut %>% select(ExperimentID,FractionID,ID,abundance_ppm)

virNOG_elut_ppm <- left_join(virNOG_elut, ppm_elut, by = c("ExperimentID","FractionID","ID"))


#Get species identifier
virNOG_elut_ppm <- virNOG_elut_ppm %>% mutate(spec = str_extract(filename, '^.....'))

#Maize is a hold out, don't use it for calculating proteins to keep
virNOG_elut_ppm_train <- virNOG_elut_ppm %>% filter(spec != "maize")

#Don't use the follow up DSSO crosslinking experiments
virNOG_elut_ppm_train <- virNOG_elut_ppm %>% filter(!grepl("DSSO", ExperimentID))

#Set up abundance and species filter
id_total_count <- virNOG_elut_ppm_train %>%

                       #Calculate total number of spectral counts per protein across all experiments
                       group_by(ID) %>%
                           mutate(ID_total = sum(Total_PeptideCount)) %>%
                           ungroup 


ids_to_keep <-  id_total_count %>%
                     select(ID, spec, ID_total) %>% 
                        unique %>%
                       #Calculate total number of species observed in
                       group_by(ID) %>%
                           mutate(specpresence = n()) %>%
                           ungroup %>% 

                       #Remove low abundance proteins
                       filter(ID_total >=32) %>%

                       #Remove proteins only seen in one species
                       filter(specpresence > 1) %>%
                       select(ID) %>%
                       unique
                       
write_csv(ids_to_keep, "IDs_to_keep_atleast32count_atleast2species.txt")

virNOG_elut_norm <- virNOG_elut_ppm %>% group_by(FractionID) %>%
                        mutate(fracnorm_PeptideCount = Total_PeptideCount/max(Total_PeptideCount)) %>%
                        ungroup %>%
                        group_by(ID, ExperimentID) %>%
                        mutate(expnorm_PeptideCount = Total_PeptideCount/max(Total_PeptideCount)) %>%
                        mutate(expnorm_ppm = abundance_ppm/max(abundance_ppm)) %>% 
                        mutate(fracexpnorm_PeptideCount = fracnorm_PeptideCount/max(fracnorm_PeptideCount)) %>%
                        ungroup
                     
#check_norms <- virNOG_elut_norm %>% select("Total_PeptideCount", "abundance_ppm", "fracnorm_PeptideCount",   
#                                           "expnorm_PeptideCount","expnorm_ppm" , "fracexpnorm_PeptideCount") %>%
#               sample_n(1000)

#One unnormalized, the other fraction normalized
#ggplot(check_norms, aes (x =Total_PeptideCount, y=abundance_ppm )) + geom_point()

#ggplot(check_norms, aes (x =Total_PeptideCount, y=expnorm_PeptideCount)) + geom_point()

#One experimentally organized, the other experimentally and fraction normalized
#ggplot(check_norms, aes (x =expnorm_PeptideCount, y=fracexpnorm_PeptideCount )) + geom_point()

#Both experimentally and fraction normalized
#ggplot(check_norms, aes (x =fracexpnorm_PeptideCount , y=expnorm_ppm )) + geom_point()

virNOG_elut_norm %>% write_csv("features/elutions_and_features/virNOG_elut_norm.csv")

virNOG_elut_filt <- virNOG_elut_norm %>% filter(ID %in% ids_to_keep$ID)



monocot <- c("wheat", "rice", "cocnu")
eudicot <- c("arath", "braol", "chqui", "sollc", "soybn", "cansa")
angiosperm <- c(monocot, eudicot)
vascular <- c(angiosperm, "selml", "cerri")
allplants <- c(vascular, "chlre")

#Need to also do each experiment-wise

#Break up by purrr on the filename. Split by filename
#Break up by purrr on the spec. Split by spec
#Then supergroup individually?

write_elution_spec <- function(df){
  if(df$spec[1] %in% c("sollc", "cocnu", "cansa", "euggr", "chqui", "maize")){
    return(0)
  }
  else{
  outfile <- paste0(
                 "features/elutions_and_features/", 
                  df$spec[1],
                  "_virNOG_elut_expnormspeccounts.wide", sep = "")

  df %>% select(FractionID, ID, expnorm_PeptideCount) %>%
    spread(FractionID, expnorm_PeptideCount, fill = 0 ) %>% 
    write_csv(outfile)
  }
}

write_elution_exp <- function(df){
  
  outfile <- paste0(
               "features/elutions_and_features/",  
               str_replace(df$filename[1], "virNOG_elut.tidy", "virNOG_elut_expnormspeccounts.wide"), sep = "")

  df %>% select(FractionID, ID, expnorm_PeptideCount) %>%
  spread(FractionID, expnorm_PeptideCount, fill = 0 ) %>% 
  write_csv(outfile)
  
}

virNOG_elut_filt %>% split(.$filename) %>% map(write_elution_exp)
virNOG_elut_filt %>% split(.$spec) %>% map(write_elution_spec)


virNOG_elut_filt %>% 
  filter(spec %in% monocot) %>% 
  select(FractionID, ID, expnorm_PeptideCount) %>%
  spread(FractionID, expnorm_PeptideCount, fill = 0 ) %>% 
  write_csv("features/elutions_and_features/monocot_virNOG_elut_expnormspeccounts.wide")
   
virNOG_elut_filt %>% 
  filter(spec %in% eudicot) %>% 
  select(FractionID, ID, expnorm_PeptideCount) %>%
  spread(FractionID, expnorm_PeptideCount, fill = 0 ) %>%
  write_csv("features/elutions_and_features/eudicot_elut_expnormspeccounts.wide")

virNOG_elut_filt %>% 
  filter(spec %in% angiosperm) %>% 
  select(FractionID, ID, expnorm_PeptideCount) %>%
  spread(FractionID, expnorm_PeptideCount, fill = 0 ) %>%
  write_csv("features/elutions_and_features/angiosperm_elut_expnormspeccounts.wide")


virNOG_elut_filt %>% 
  filter(spec %in% vascular) %>% 
  select(FractionID, ID, expnorm_PeptideCount) %>%
  spread(FractionID, expnorm_PeptideCount, fill = 0 ) %>%
  write_csv("features/elutions_and_features/vascular_elut_expnormspeccounts.wide")

virNOG_elut_filt %>% 
  filter(spec %in% allplants) %>% 
  select(FractionID, ID, expnorm_PeptideCount) %>%
  spread(FractionID, expnorm_PeptideCount, fill = 0 ) %>%
  write_csv("features/elutions_and_features/allplants_elut_expnormspeccounts.wide")


#Hypergeometric files

write_hypergeo_exp <- function(df){

  
    outfile <- paste0(
      "features/hypergeo_elutions/min5_",  
      str_replace(df$filename[1], "virNOG_elut.tidy", "virNOG_elut.tidy"), sep = "")
    
    df %>% 
        select(ID, FractionID) %>%
        write_csv(outfile)
}

write_hypergeo_spec <- function(df){
   if(df$spec[1] %in% c("sollc", "cocnu", "cansa", "euggr", "chqui")){
       return(0)
   }
   else{
    outfile <- paste0(
    "features/hypergeo_elutions/min5_", 
    df$spec[1],
    "_virNOG_elut.tidy", sep = "")
  df %>% 
    select(ID, FractionID) %>%
    write_csv(outfile)
   }
}

virNOG_elut_filt_threshold <- virNOG_elut_filt %>% filter(Total_PeptideCount >=5)
virNOG_elut_filt_threshold %>% split(.$filename) %>% map(write_hypergeo_exp)
virNOG_elut_filt_threshold %>% split(.$spec) %>% map(write_hypergeo_spec)

virNOG_elut_filt_threshold %>% 
  filter(spec %in% monocot) %>% 
  select(ID, FractionID) %>%
  write_csv("features/hypergeo_elutions/min5_monocot_virNOG_elut.tidy")

virNOG_elut_filt_threshold %>% 
  filter(spec %in% eudicot) %>% 
  select(ID, FractionID) %>%
  write_csv("features/hypergeo_elutions/min5_eudicot_virNOG_elut.tidy")

virNOG_elut_filt_threshold %>% 
  filter(spec %in% angiosperm) %>% 
  select(ID, FractionID) %>%
  write_csv("features/hypergeo_elutions/min5_angiosperm_virNOG_elut.tidy")

virNOG_elut_filt_threshold %>% 
  filter(spec %in% vascular) %>% 
  select(ID, FractionID) %>%
  write_csv("features/hypergeo_elutions/min5_vascular_virNOG_elut.tidy")

virNOG_elut_filt_threshold %>% 
  filter(spec %in% allplants) %>% 
  select(ID, FractionID) %>%
  write_csv("features/hypergeo_elutions/min5_allplants_virNOG_elut.tidy")



